package farm.ants.agent;
import farm.simulator.*;
import farm.metagent.*;
import farm.ants.plugins.*;
import java.util.*;
/**
 */
public class OptimalAgent extends Agent {

    // Constants
    public static final String PROP_OPTIMALUTILITY = "AgentOptimalUtility";
    public static final String PROP_OPTIMALTRACKS = "AgentOptimalTracks";
    boolean computed = false;
    int slotCount = 3;

    /**
     * Constructor
     * @param metagent A handle to the metagent this agent belongs to
     * @param name The agents name
     */
    public OptimalAgent(FarmMetAgent metagent, String name) {
	super(metagent, name);
 	setProperty(PROP_OPTIMALUTILITY, new Float(0));
	setProperty(PROP_OPTIMALTRACKS, new Integer(0));
    }


    /**
     * Called by the metagent to grant run time to this agent.
     * The SensorAgent simple pulls any incoming messages
     * off of its incoming message queue and if they are a rebing request
     * changes it local schedule and sends update messages to registered 
     * track managers.
     */
    public void pulse() {
	super.pulse();
        //checkRun();

	if(!computed){
	    computed = computeOptimal();
	}
	endRun();
    }


    private boolean computeOptimal(){

	//create the list of managers for the function call
	Integer targInt = (Integer) getProperty(FarmDriver.CLA_TARGETS);
	if(targInt == null) return false;
	int sensors = ((Integer) getProperty(FarmDriver.CLA_SENSORS)).intValue();

	int targets = targInt.intValue();
	Vector mans = new Vector();
	float total = 0.0f;
	int totTrack = 0;
	Hashtable fSensInfo = new Hashtable();
	for(int i = 0;i<targets;i++){
	    checkRun();
	    String manName = "M" + i;
	    StringTokenizer tok = new StringTokenizer((String) getProperty(manName + ":res"), ",");
	    TMConflict TMC = new TMConflict(manName);
	    int count =0;
	    while(tok.hasMoreTokens() && count < 4){
		checkRun();
		String data = tok.nextToken();
		String sensName = data.substring(0, data.indexOf(" "));
		TMC.addConflict(sensName);
		if(!fSensInfo.containsKey(sensName)){
		    SlotSchedule SS = new SlotSchedule();
		    fSensInfo.put(sensName, SS);
		}
		count ++;
	    }
	    TMC.setNumAlts(TMC.numLocalConflicts());
	    mans.add(TMC);
	}
	log.log("Before cleaning " + mans.toString(), log.LOG_INFO);
	sensors = fSensInfo.size();
	//clean up the TMConflicts
	for(int i = 0; i<mans.size();i++){
	    checkRun();
	    TMConflict t1 = (TMConflict) mans.elementAt(i);
	    for(Iterator k = t1.getConflicts().iterator();k.hasNext();){
		boolean located = false;
		String thisConflict = (String) k.next();
		for(int j = 0; j<mans.size();j++){
		    checkRun();
		    if(i==j) continue;
		    TMConflict t2 = (TMConflict) mans.elementAt(j);
		    if(t2.hasConflict(thisConflict)){
			located = true;
			break;
		    }
		}
		if(!located){
		    k.remove();
		}
	    }
	}

	//clean up the fSensInfo list
	fSensInfo.clear();
	for(int i = 0; i<mans.size();i++){
	    checkRun();
	    TMConflict t1 = (TMConflict) mans.elementAt(i);
	    for(Iterator k = t1.getConflicts().iterator();k.hasNext();){
		String sensName = (String) k.next();
		if(!fSensInfo.containsKey(sensName)){
		    SlotSchedule SS = new SlotSchedule();
		    fSensInfo.put(sensName, SS);
		}
	    }
	}
	log.log("After cleaning " + mans.toString(), log.LOG_INFO);
	setStartObjs(mans);

	Vector bestAssign = null;
	
	boolean found = false;
	SolutionSet SS = new SolutionSet();
	//System.out.println("TConflict: " + mans.toString());
	
	float currBest = 0;
	while(true){
	    checkRun();
	    //first let's do a sanity check, can we every expect
	    //to get a solution given the current objective levels
	    //and the sensors
	    if(solutionPossible(mans, sensors)){
		log.log("Generating solutions", log.LOG_INFO);
		found = genGlobalSolSet(schedulesToArray(fSensInfo),new Vector(fSensInfo.keySet()), mans, 0, SS, new Solution());
	    }
	    
	    if(found){
		//store results
		currBest = calculateUtility(mans);
		log.log("Found a good one", log.LOG_INFO);
		bestAssign = cloneVector(mans);
		setProperty(PROP_OPTIMALUTILITY, new Float(calculateUtility(bestAssign)));
		setProperty(PROP_OPTIMALTRACKS, new Integer(calculateTotTracks(bestAssign)));	    
	    }
	    boolean okay = false;
	    while(lowerObjective(mans)){
		checkRun();
		if(currBest >= calculateUtility(mans)){
		    continue;
		}
		else{
		    log.log("Trying this one. Utility: " + calculateUtility(mans), log.LOG_INFO);
		    found = false;
		    okay = true;
		    break;
		}
	    }
	    if(!okay){
		log.log("!!!!Search finished!!!!!", log.LOG_INFO);
		break;
	    }
	}

	log.log("Found the answer!!!!!! Util: " + calculateUtility(bestAssign) + " Tracks: " +  calculateTotTracks(bestAssign), log.LOG_INFO);
	log.log("Final mans " + bestAssign.toString(), log.LOG_INFO);
	setProperty(PROP_OPTIMALUTILITY, new Float(calculateUtility(bestAssign)));
	setProperty(PROP_OPTIMALTRACKS, new Integer(calculateTotTracks(bestAssign)));
	return true;
    }

    private int calculateTotTracks(Vector mans){
	int totalTracks = 0;
	for(Iterator i = mans.iterator();i.hasNext();){
	    TMConflict TMC = (TMConflict) i.next();
	    ObjEntry objE= TMC.getObjFunc().getCurrObj();
	    if(objE.numAgents() >2)
		totalTracks++;
	}
	return totalTracks;
    }

    private int calculateTrackable(Vector mans){
	int totalTracks = 0;
	for(Iterator i = mans.iterator();i.hasNext();){
	    checkRun();
	    TMConflict TMC = (TMConflict) i.next();
	    if(TMC.getNumAlts() >2)
		totalTracks++;	    
	}
	return totalTracks;
    }

    private float calculateUtility(Vector mans){
	float totalUtil = 0.0f;
	for(Iterator i = mans.iterator();i.hasNext();){
	    checkRun();
	    TMConflict TMC = (TMConflict) i.next();
	    ObjEntry objE= TMC.getObjFunc().getCurrObj();
	    if(objE.numAgents() >2)
		totalUtil += objE.numSlots() * (float) Math.sqrt(objE.numAgents()-2);
	    //log.log(objE.toString() + " " + objE.numSlots() * (float) Math.sqrt(objE.numAgents()-2) + " " + totalUtil, log.LOG_INFO);
	}
	return totalUtil - (calculateTrackable(mans) - calculateTotTracks(mans));
    }

    boolean solutionPossible(Vector managers, int sensors){
	int totAvail=sensors*SlotSchedule.NUMSLOTS;
	int totDes=0;

	for(int i=0;i<managers.size();i++){
	    checkRun();
	    TMConflict thisTM = (TMConflict) managers.elementAt(i);
	    ObjEntry obj = thisTM.getObjFunc().getCurrObj();
	    totDes+=(obj.numAgents()*obj.numSlots());
	}
	return totAvail>=totDes;
    }


    /**
     * generateConflictSol is responsible for, given sensor schedules,
     * determining if a non-conflicting solution exists on all or a subset
     * of the sensor and slots that are unser consideration
     * If there is then it returns a vector of all of the non-conflicting
     * solutions if none, then it return null
     * <p>
     * There are several ways to use this function that have value
     *
     * 1.  If the sensors (in schedules) have elements that are 
     * occupied, this function will take that into account.  
     * This can be used to generate the set of acceptable 
     * solutions for local resolution
     * In this use, you must specify that the availSensors = 
     * schedules.size() in order to get solutions for all of
     * the sensors
     *
     * 2.  If you want to generate the set of acceptable
     * solutiond for a set of sensors and want it to be
     * non-respective of the sensors schedules then create
     * a hashtable with the agent names as keys and having
     * empty slot schedules as elements.  Again you must
     * specify the availSensors = schedules.size()
     *
     * 3.  If you want to just get the the possible solutions
     * for 2 out of 4 sensors then schedules should only
     * contain the names and schedules of the 2 sensors you want
     * schedules for.  The availSensors should be set to 4 in this
     * case.  This will generate the combinations respective of the
     * schedules you have listed.
     *
     *
     * @param schedules key: sensore name  element: SlotSchedule
     * @param obj The current objective level
     * @return Vector (solutions) elements: Vector (slots) elements: 
     *  Vector (sensors) elements: agent names
     *     
     */
    Vector generateConflictSol(int[][] schedMap, Vector sensNames, ObjEntry obj, int availSensors) {


	//special case, he doesn't want anything
	if(obj.numAgents() * obj.numSlots() == 0){
	    Vector solution = new Vector();
	    Vector thisSoln = new Vector();
	    for(int i=0;i<SlotSchedule.NUMSLOTS;i++)
		thisSoln.add(new Vector());

	    solution.add(thisSoln);
	    return solution;
	}

	//invert the schedules
	Vector slotInfo = toSlotMajor(schedMap, sensNames);

	int goodSlots = 0;
	Vector goodStuff = new Vector();
	for(int i =0;i<slotInfo.size();i++){
	    Vector tmpVect = (Vector) slotInfo.elementAt(i);
	    int count = tmpVect.size();
	    //log.log("slot i: " + i + " count: " + count + " data: " + tmpVect.toString(), Log.LOG_DEBUG);

	    if(availSensors-(sensNames.size()-count) >= obj.numAgents()){
		goodStuff.add(new Integer(i));
		goodSlots++;
	    }
	}

        checkRun();
	if(goodSlots >= obj.numSlots()){
	    //great, we know there is at least one good solution
	    //log.log("Found at least one good solution.  Generating combinations", Log.LOG_SPAM);

	    //first, let's get the list of combinations of good
	    //slots, this is used in generation of the legal solutions
	    
	    Vector slotCombs = Choose(goodStuff, obj.numSlots());


	    //slotComb now has vectors of all possible slot combinations 
	    //which meet the criteria for a solution

	    //now we need for each of those slots, the legal combinations
	    //of agents

	    Vector sensCombs = new Vector();	
	    for(int j=0;j<slotInfo.size();j++){
		//does this slot have enough to even consider it as a 
		//a part of a viable solution?
		checkRun();
		int numSensInSlot = ((Vector) slotInfo.elementAt(j)).size();
		int availForSlot = availSensors - (sensNames.size() - numSensInSlot);
		int lb = Math.max(0, obj.numAgents()-
				availForSlot+numSensInSlot);
		int ub = Math.min(numSensInSlot, obj.numAgents());
		//log.log("lb " + lb+ " ub " + ub, Log.LOG_SPAM);
		if(lb <= ub){
		    //the slot can generate at least one good combination
		    checkRun();		    
		    Vector slotSensCombs = new Vector();
		    //for each of the slots generate the useable 
		    //sensor combination
		    
		    for(int i=lb;i<=ub;i++){
		    //generate all combinations for the n sensors such that
		    //we choose i
			Vector sensComb = Choose((Vector) slotInfo.
						 elementAt(j),i);
			slotSensCombs.addAll(sensComb);
		    }
		    sensCombs.add(slotSensCombs);
		}
		else{
		    sensCombs.add(new Vector());
		}
	    }

	    //for(int i=0;i<sensCombs.size();i++){
	    //	log.log("Combination " + i + " is " + ((Vector) sensCombs.elementAt(i)).toString(), Log.LOG_SPAM);
	    //}


	    //now we nee to construct the final answer
	    //for each slot combination, we need to pair wise match the slot 
	    //combinations in the slots
	    Vector solution = new Vector();
	    
	    for(int i =0;i<slotCombs.size();i++){
		//get one of the slot combinations
		Vector slotComb = (Vector) slotCombs.elementAt(i);
		checkRun();		
		//log.log("Evaluating slot combination " + i + ": " + slotComb.toString(), Log.LOG_SPAM);


		//for this slot combination we need to construct
		//a temporary sensory combination list with only the
		//slots in the combination
		//for all the other slots, we need to add a record
		//so the table join works correctly
		
		Vector tmpSensCombs = new Vector();
		//fill tmpSensCombs with empty vectors
		for(int j =0;j<sensCombs.size();j++){
		    tmpSensCombs.add(new Vector());
		}
		checkRun();		
		for(int j =0;j<slotComb.size();j++){
		    int slotNum = ((Integer) slotComb.elementAt(j)).intValue();
		    tmpSensCombs.setElementAt(sensCombs.elementAt(slotNum), slotNum);
		    //log.log("Adding slot "+slotNum+" which is " + 
		    //    ((Vector) sensCombs.elementAt(slotNum)).toString()
		    //    + " to tmpSensCombs", Log.LOG_SPAM);
		}

		Vector tmpsol = tableJoin(tmpSensCombs);

		solution.addAll(tmpsol);
	    }

	    return solution;
	}
	return new Vector();
    }

    Vector tableJoin(Vector tables){
	Vector retData = new Vector();
	maxJoin(tables, 0, new Vector(), retData);
	return retData;
    }

    void maxJoin(Vector tables, int currdepth, Vector solution, Vector Sets){
	//this function computes the pair wise join of all of the vectors
	//in the vector table.  It returns tha answer as a vector of vectors

	Vector thisTable = (Vector) tables.elementAt(currdepth);
	int maxdepth = tables.size();
        checkRun();
	if(thisTable.size() == 0){
	    solution.add(new Vector());
	    if(currdepth < maxdepth-1){
		maxJoin(tables, currdepth +1, solution, Sets);
		solution.remove(currdepth);
	    }
	    else{
		Sets.add(new Vector(solution));
		solution.remove(currdepth);
	    }	
	}
	else{
	    for(int i=0;i<thisTable.size();i++){
		solution.add(thisTable.elementAt(i));
		if(currdepth < maxdepth-1){
		    maxJoin(tables, currdepth +1, solution, Sets);
		    solution.remove(currdepth);
		}
		else{
		    Sets.add(new Vector(solution));
		    solution.remove(currdepth);
		}
	    }
	}
    }

    Vector Choose(Vector elements, int r){
	//this function takes a vector of n elements and returns a vector
	//that contains the nCr possible combinations (vectors) of the elements

	Vector retData = new Vector();
	if(r>0){
	    nCr(r, new Vector(), 0, elements, retData);
	}
	else{
	    retData.add(new Vector());
	}
	return retData;
    }
	

    void nCr(int maxdepth, Vector solution, int first, 
	     Vector elements, Vector Sets){

	int currdepth = solution.size();
	int last = elements.size();
        checkRun();
	for(int i = first;i<= last-maxdepth+currdepth;i++){
	    
	    solution.add(elements.elementAt(i));
	    if(currdepth < maxdepth-1){
		nCr(maxdepth, solution, i+1, elements, Sets);
		solution.remove(currdepth);
	    }
	    else{
		//Operate on solution
		Sets.add(new Vector(solution));
		solution.remove(currdepth);
	    }
	}
    }

    Hashtable toSensorMajor(Vector slotData, Vector sensors, String manData){
	/*this function returns a hashtable with each key being a sensor
	  name and each element being a SlotSchedule

	  If you specify names in a vector, the hashtable that is returned
	  contain one element per name.  If the name was not in slotdata
	  the slotschedule associated with the sensor will be empty

	  coverts        
	
       	  Slot 1 S1, S3
	  Slot 2 S2, S3
	  Slot 3 S1, S3
	  
	  to

	  S1    None  T1   None
	  S2    T1   None   T1
	  S3    None None  None

	*/

	Hashtable tmpHash = new Hashtable();
 	for(int i=0;i<sensors.size();i++){
	    SlotSchedule SS = new SlotSchedule();
	    tmpHash.put(sensors.elementAt(i),SS);
	}
        checkRun();	
	for(int i=0;i<slotData.size();i++){
	    Vector names = (Vector) slotData.elementAt(i);
	    for(int j = 0;j<names.size();j++){
		SlotSchedule SS;
		if(tmpHash.containsKey(names.elementAt(j))){
		    //update the current slot schedule for that
		    //sensor
		    SS = (SlotSchedule) 
			tmpHash.get(names.elementAt(j));
		    
		}
		else{
		    //create a new entry in the hashtable and
		    //set the slot schedule correctly
		    SS = new SlotSchedule();
		    tmpHash.put(names.elementAt(j), SS);
		}
		checkRun();
		SS.add(i,manData);
	    }
	}
	return tmpHash;
    }
    public boolean genGlobalSolSet(int[][] constraints, Vector sensNames, Vector managers, int manNum, SolutionSet SS, Solution thisSoln){
	
	/*generate the set of feasible solutions given the contraints
	  for each of these figure out if the next track manager
	  has acceptable solutions.  If they don't prune
    
	Hashtable (constraints)
	    |
	    keys: Sensor Names
            elements: SlotSchedule

	    
        Vector (managers, order by constraints)
           |
	   elements: TMConflict

	int (manNum): the manager under consideration in the recursion


	Hashtable globalSolns  //holds the set of partial solutions that 
            |                  //lead to globally feasible solutions
	    keys: Managers names
	    elements: Vector(solutions)
	                |
			elements: Vector(slots)
			            |
				    elements: Vector(sensors)
				                |
						elements:sensor names

	*/

	Hashtable schedules = new Hashtable();
	TMConflict thisTMC = (TMConflict) managers.elementAt(manNum);
	boolean foundOne = false;

	//create a Vector with only the contraints for agents that this
	//manager has in conflict
	Vector thisConflicts = thisTMC.getConflicts();
	int[][] schedMap = new int[thisConflicts.size()][SlotSchedule.NUMSLOTS];
	for(int i = 0;i<thisConflicts.size();i++){
	    for(int j = 0;j<SlotSchedule.NUMSLOTS;j++){
		schedMap[i][j] = constraints[sensNames.indexOf(thisConflicts.elementAt(i))][j];
	    }
	}
	
	//given these constraints, generate the set of potential 
	//partial solutions for this track manager
	Vector partSol = generateConflictSol(schedMap, thisConflicts, thisTMC.getObjFunc().getCurrObj(), thisTMC.getNumAlts());


	//if we have none, the return and say that we're over constrained
	if(partSol.size() == 0){
	    //too constrained to make a solution
	    return false;
	}
        checkRun();
	//good, we have solutions at this level to evaluate
	if(manNum < managers.size()-1){
	    for(int i = 0;i<partSol.size();i++){
		//long startTime = System.currentTimeMillis();

		//construct the constraint set for the next lower level	
		addConstraints(constraints, sensNames, 
						       (Vector) partSol.
						       elementAt(i));
		
		ResourceSchedule RS = new ResourceSchedule();
		RS.set((Vector) partSol.elementAt(i));
		thisSoln.put(thisTMC.getName(), RS);
		if(genGlobalSolSet(constraints,sensNames, managers, manNum + 1, SS, thisSoln)){
		    //there were solutions at the next lower level
		    //add this partial solution to my list of global
		    //solutions.
		    foundOne = true;
		    break;
		}
		removeConstraints(constraints, sensNames, 
						       (Vector) partSol.
						       elementAt(i));
		thisSoln.remove(thisTMC.getName());
	    }
	}
	else{
	    //go ahead and add all of the solutions to the partialSolns set
	    foundOne = true;
	    //make sure we update the global solution map
	    checkRun();
	    for(int i= 0;i<partSol.size();i++){
		ResourceSchedule RS = new ResourceSchedule();
		RS.set((Vector) partSol.elementAt(i));
		thisSoln.put(thisTMC.getName(), RS);
		//System.out.println("Adding solution: " + thisSoln.toString());
		SS.addSolution((Solution) thisSoln.clone());
		thisSoln.remove(thisTMC.getName());
	    }
	}
	return foundOne;
    }


    void addConstraints(int[][] oldConsts, Vector sensNames, Vector newConsts){
        checkRun();
	for(int i =0;i<newConsts.size();i++){
	    Vector thisSlot = (Vector) newConsts.elementAt(i);
	    for(int j=0;j<thisSlot.size();j++){
		String thisAgent = (String) thisSlot.elementAt(j); 
		oldConsts[sensNames.indexOf(thisAgent)][i]++;
	    }   
	}
    }

    void removeConstraints(int[][] oldConsts, Vector sensNames, Vector newConsts){
        checkRun();
	for(int i =0;i<newConsts.size();i++){
	    Vector thisSlot = (Vector) newConsts.elementAt(i);
	    for(int j=0;j<thisSlot.size();j++){
		String thisAgent = (String) thisSlot.elementAt(j); 
		oldConsts[sensNames.indexOf(thisAgent)][i]--;
	    }   
	}
    }

    /**
     * This function converts the map and the names into
     * a list for each slot of available resources
     */
    Vector toSlotMajor(int[][] schedMap, Vector names){
	//this function returns a vector of the slots where each slot is
	//a vector that contains the names of the sensors that have the slot
	//available

	/*converts        
	
	  S1    None  T1   None
	  S2    T2   None   T2
	  S3    None None  None


	  to

	  Slot 1 S1, S3
	  Slot 2 S2, S3
	  Slot 3 S1, S3
	
	*/

	Vector slotData = new Vector(SlotSchedule.NUMSLOTS);
	//count the columner counts
	for(int i = 0;i<SlotSchedule.NUMSLOTS;i++){
	    Vector sens = new Vector();
	    for(int j = 0;j<names.size();j++){
		String sensName = (String) names.elementAt(j);
		if(schedMap[j][i] == 0){
		    //free slot
		    sens.add(sensName);
		}
	    }
	    slotData.add(sens);
	}
	return slotData;
    }

    public int[][] schedulesToArray(Hashtable scheds){
	int[][] schedMap = new int[scheds.size()][SlotSchedule.NUMSLOTS];
	Vector names =  new Vector(scheds.keySet());
	for(int i = 0;i<names.size();i++){
	    String name = (String) names.elementAt(i);
	    SlotSchedule SS = (SlotSchedule) scheds.get(name);
	    for(int j = 0;j<SlotSchedule.NUMSLOTS;j++){
		schedMap[i][j] = SS.numCommitments(j);
	    }
	}
	return schedMap;
    }

    private void resetBelowMe(Vector mans, int start, int finish){
	for(int i = start;i<finish;i++){
	    ((TMConflict) mans.elementAt(i)).
		getObjFunc().highestObj();
	    setStartObj((TMConflict) mans.elementAt(i));
	} 
    }


    /**
     * Finds the manager who is causing the over-constrained condition
     * and lower his objective function
     * @param managers A Vector of the TMConflicts for the managers in the negotiation
     * @return True if one of the managers was lowered
     */
    boolean lowerObjective(Vector managers){

	//The method for doing this is simple, just find the
	//manager with both the highest level and that cannot meet
	//his needs with his own non con sensors and lower his level
	int man = findNextEligible(managers,0);
        checkRun();
	while(man != -1){
	    TMConflict thisTM = (TMConflict) managers.elementAt(man);
	    if(thisTM.getObjFunc().lowerObj()){
		resetBelowMe(managers, 0, man);
		printObjs(managers);
		return true;
	    }
	    else{
		//reset mine
		man = findNextEligible(managers,man+1);
	    }
	}
	return false;
    }
    


    private int findNextEligible(Vector managers, int index){
        checkRun();
	for(int i=index;i<managers.size();i++){
	    TMConflict thisTM = (TMConflict) managers.elementAt(i);
	    ObjEntry obj = thisTM.getObjFunc().getCurrObj();
	    int numOwn = thisTM.getNumAlts() - thisTM.numLocalConflicts();
	    //check if eligible
	    if(obj.numAgents() > numOwn){
		//the first, try to lower his 
		return i;
	    }
	}
	return -1;
    }
    
    private Vector cloneVector(Vector vect){
	Vector ret = new Vector();
	for(int i=0;i<vect.size();i++){
	    ret.add(((TMConflict) vect.elementAt(i)).clone());
	}
	return ret;
    }

    private void printObjs(Vector mans){
	String outdata = "";
	for(Iterator i = mans.iterator();i.hasNext();){
	    TMConflict TMC = (TMConflict) i.next();
	    ObjEntry objE= TMC.getObjFunc().getCurrObj();
	    outdata = outdata + "Man: " + TMC.getName() + " " +  objE.toString() + ", ";
	}
	log.log(outdata, log.LOG_INFO);
    }

    private void setStartObjs(Vector mans){ 
	for(Iterator i = mans.iterator();i.hasNext();){
	    TMConflict TMC = (TMConflict) i.next();
	    setStartObj(TMC);
	}
    }

    private void setStartObj(TMConflict TMC){
	ObjFunction objective = TMC.getObjFunc();
	//make sure that we can start our objective level at something
	//that has the posibility of being achieveble
	ObjEntry objE = objective.getCurrObj();
	while(objE.numAgents() > TMC.getNumAlts()){
	    checkRun();
	    objective.lowerObj();
	    objE = objective.getCurrObj();
	}  
    }
}
