package farm.ants.agent;
import java.util.*;

/**
 * An object used for locking in the PulseActionController
 * <p>
 * Migrated to farm from ANTS 14 Jul 2002
 *
 * @author Roger Mailler
 */
public class PACLock implements Comparable {
    
    String name;
    int time;
    int breaker;
    int priority = 0;  //how important is this lock
    int Requester =0;

    /**
     * Constructor
     * @param r the requester ID of the lock
     */
    public PACLock(int r){
	Requester = r;
    }

    /**
     * Constructor
     * @param r the requester ID of the lock
     * @param n the name of the lock being requested
     * @param t the time the request is being made
     */
    public PACLock(int r, String n, int t) {
	Random rnd = new Random();
	Requester = r;
	name = n;
	time = t;
	breaker = rnd.nextInt();
    }

    /**
     * Gets the lock requester
     * @return the ID of the requester
     */
    public int getRequester(){return Requester;}

    /**
     * Sets the request ID of the lock
     * @param r the ID
     */
    public void setRequester(int r){Requester = r;}
    
    /**
     * Gets the time the lock was requested
     * @return the time the lock request was made
     */
    public int getTime(){return time;}

    /**
     * Sets the request time if the lock
     * @param t the time to set the lock request to
     */
    public void setTime(int t){time = t;}
    
    /**
     * Gets the ID used to break ties for the comparator
     * @return the ID used to break ties for the lock
     */
    public int getLID(){return breaker;}

    /**
     * Sets the ID used to break ties in the comparator
     * @param i the tie breaker
     */
    public void setLID(int i){breaker = i;}
    
    /**
     * Gets the name of the lock
     * @return the name of the lock
     */
    public String getName(){return name;}
    
    /**
     * Sets the name for the lock
     * @param n the name
     */
    public void setName(String n){name = n;}

    /**
     * Sets the requester priority for the lock
     * @param i the priority
     */
    public void setPriority(int i){priority = i;}
    
    /**
     * Gets the requester priority for the lock
     * @return the priority
     */
    public int getPriority(){return priority;}

    /**
     * Comparator
     * Orders by priority, time, then LID
     * @param o1 the object to compare this one to
     * @return -1 if this element should be in front of the other in the order
     */
    public int compareTo(Object o1){
	PACLock PACL = (PACLock) o1;
	if(PACL.getPriority() < this.getPriority()){
	    return -1;
	}
	else if (PACL.getPriority() > this.getPriority()){
	    return 1;
	}

	if(PACL.getTime() > this.getTime()){
	    return -1;
	}
	else if(PACL.getTime() < this.getTime()){
	    return 1;
	}
	else{
	    if(PACL.getLID() > this.getLID()){
		return -1;
	    }
	    else if(PACL.getLID() < this.getLID()){
		return 1;
	    }
	    else{
		return 0;
	    }
	}
    }

    /**
     * Returns a string describing the lock
     * @return a string description of the object
     */
    public String toString(){
	return "Requester: "+ Requester + " name: " + name  + " priority: " + priority +" time: " + time +"  ID: " + breaker;
    }
    
} // PACLock
