package farm.ants.agent;
import java.util.*;

/**
 * This class holds the resource usage schedule for a particular manager
 * @author Roger Mailler
 */
public class ResourceSchedule {
    /**
     * Number of slots in the schedule
     */
    public static final int NUMSLOTS = 3;
    ArrayList slots[] = new ArrayList[NUMSLOTS];
    
    /**
     * Constructor
     */
    public ResourceSchedule() {
	for(int i = 0;i<NUMSLOTS;i++){
	    slots[i] = new ArrayList();
	}
    }
    
    /**
     * Adds a resource to a particular slot
     * @param slot The slot
     * @param res The name of the resource
     */
    public void add(int slot, String res){
	int here = 0;
	for (int i = 0;i<slots[slot].size();i++){
	    if(((String) slots[slot].get(i)).compareTo(res) > 1)
		break;
	    here++;
	}
	slots[slot].add(here, res);
	
    }

    /**
     * Removes a particular resource use from a slot
     * @param slot The slot
     * @param res The name of the resource
     */
    public void remove(int slot, String res){
	slots[slot].remove(res);
    }

    /**
     * Gets the contents of a slot
     * @param slot The slot
     */
    public ArrayList getSlot(int slot){
	return slots[slot];
    }

    /**
     * Clears the contents of a slot
     * @param slot The slot
     */
    public void clearSlot(int slot){
	slots[slot].clear();
    }


    //temporary
    public void set(Vector vals){
	for(int i = 0;i<NUMSLOTS;i++){
	    Vector slot = (Vector) vals.elementAt(i);
	    for(int j=0;j<slot.size();j++){
		add(i,(String) slot.elementAt(j));
	    }
	}	
    }

    /**
     * Returns a string representation of the schedule
     * @return The string
     */
    public String toString(){
	String data = "[";	
	for(int i = 0;i<NUMSLOTS;i++){
	    String slotData = "[";
	    ArrayList slot = slots[i];
	    for(int j=0;j<slot.size();j++){
		slotData = slotData + (String) slot.get(j) 
		    + ((j<slot.size()-1)?",":"");
	    }

	    data = data + slotData + "]" + ((i<NUMSLOTS -1)?", ":"");
	}
	return data + "]";
    }

    /**
     * Converst the resource schedule into a Hashtable
     * of slot schedules (one element for each resource)
     * @param manName The name of the manager who's schedule this is
     * @return Hashtable of SlotSchedules keyed on resource name
     */
    public Hashtable toSlotSchedules(String manName){
	Hashtable ret = new Hashtable();
	
	for(int i = 0;i<NUMSLOTS;i++){
	    ArrayList slot = slots[i];
	    for(int j=0;j<slot.size();j++){
		SlotSchedule SS;
		String name = (String) slot.get(j);
		if(ret.containsKey(name)){
		    SS = (SlotSchedule) ret.get(name);
		}
		else{
		    SS = new SlotSchedule();
		    ret.put(name, SS);
		}
		SS.add(i, manName);
	    }
	}
	return ret;
    }



} // ResourceSchedule
