package farm.ants.plugins;
/**
 * FarmWorldPanel.java
 *
 *
 * Created: Wed Jul 10 14:47:27 2002
 *
 * @author Roger Mailler
 * @version
 */

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.*;
import java.awt.geom.*;
import farm.ants.agent.SlotSchedule;

public class FarmWorldPanel extends JPanel
    implements MouseListener {

    FarmDisplayFrame FDF = null;
    int panWidth, panHeight;
    int envWidth, envHeight;
    Rectangle viewPort;
    Color targColor = Color.red;
    Color sensColor = Color.black;
    Color backColor = Color.gray;
    Color negColor = Color.blue;
    Color waitColor = Color.green;
    Image backGround = null;
    Image scaleBar = null;
    boolean backSens = false;
    int zoomLevel = 1;
    int baseSensSize = 6;
    Dimension backSize = new Dimension();

    // Preferences
    boolean showtracks = true;
    boolean shownegot = true;
    boolean targettext = true;
    boolean sensortext = false;

    public FarmWorldPanel(FarmDisplayFrame fdf) {
	FDF = fdf;
	panWidth = 600;
        panHeight = 600;
	viewPort = new Rectangle(0,0,panWidth, panHeight);
	addMouseListener(this);
    }

    public void setShowTracking(boolean b) { showtracks = b; }
    public boolean getShowTracking() { return showtracks; }
    public void setShowNegotiations(boolean b) { shownegot = b; }
    public boolean getShowNegotiations() { return shownegot; }
    public void setShowTargetText(boolean b) { targettext = b; }
    public boolean getShowTargetText() { return targettext; }
    public void setShowSensorText(boolean b) { sensortext = b; backSens = false; }
    public boolean getShowSensorText() { return sensortext; }

    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {
	
	if((e.getModifiers() & e.BUTTON1_MASK) !=0){
	    Dimension d = getSize();
	    float relOrgX = (float) e.getPoint().x/(float)d.width;
	    float relOrgY = (float) e.getPoint().y/(float)d.height;
	    //System.err.println("click " + e.getPoint().x + " relOrgX " + relOrgX);

	    if(zoomLevel < 32) {
		zoomLevel *=2;
		float newOrgX = (float) (viewPort.getX() + viewPort.getWidth() * relOrgX);
		float newOrgY = (float) (viewPort.getY() + viewPort.getHeight() * relOrgY);
		float newEnvWidth = (float) envWidth/(float) zoomLevel;
		float newEnvHeight = (float) envHeight/(float) zoomLevel;

		newOrgX -=  newEnvWidth/2f;
		newOrgY -= newEnvHeight/2f;
		
		//check the bounds
		if(newOrgX <0) newOrgX =0f;
		if(newOrgX +  newEnvWidth > envWidth) 
		    newOrgX = envWidth- newEnvWidth;
		if(newOrgY <0) newOrgY =0f;
		if(newOrgY +  newEnvHeight > envHeight) 
		    newOrgY = envHeight- newEnvHeight;

		invalidate();
		FDF.getContentPane().validate();
	    }

	}
	else if ((e.getModifiers() & e.BUTTON3_MASK) !=0){
	    if(zoomLevel > 1){
		zoomLevel /=2;
		float newOrgX =  (float) (viewPort.getX() - viewPort.getWidth()/2.0);
		float newOrgY = (float) (viewPort.getY() - (float) viewPort.getHeight()/2.0f);

		float newEnvWidth = (float) envWidth/(float) zoomLevel;
		float newEnvHeight = (float) envHeight/(float) zoomLevel;

		//check the bounds
		if(newOrgX <0) newOrgX =0f;
		if(newOrgX +  newEnvWidth > envWidth) 
		    newOrgX = envWidth- newEnvWidth;
		if(newOrgY <0) newOrgY =0f;
		if(newOrgY +  newEnvHeight > envHeight) 
		    newOrgY = envHeight- newEnvHeight;

		invalidate();
		FDF.getContentPane().validate();
	    }
	}
	//System.err.println("zoom " + zoomLevel);
	//System.err.println("port " + viewPort.toString());
    }
    public void setEnvWidth(int w){
	envWidth = w;
	viewPort.setSize(w, (int) viewPort.getHeight());
    }
    public void setEnvHeight(int h){
	envHeight =h;
	viewPort.setSize((int)viewPort.getWidth(), h);
    }

    public void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D)g;
	super.paintComponent(g);
	Color c = g.getColor();
	Dimension d = getSize();
	if (envWidth == 0) return;
	Rectangle bounds = FDF.getJSP().getViewport().getViewRect();

        // Get some metrics
        FontMetrics metrics = g.getFontMetrics();
        float fontheight = metrics.getAscent();

        // What's the scale?
	float scalex = (float)d.width / (float)envWidth;
	float scaley = (float)d.height / (float)envHeight;
        
        // Determine scale bar dimensions
        float world2env = (float)envWidth/ (float)d.width;
        float boundsSize = (float) bounds.width*0.2f;
        int precision = ((int) (Math.log(envWidth)/Math.log(10.0))-2);
        float scaleVal = Closest(boundsSize * world2env, precision);

        // Draw the scale bar
	if (scaleBar == null || bounds.width != viewPort.width || !backSize.equals(d)) {
	    float sizeX = scaleVal/world2env;  //size of the window
	    int xInset = 5;
	    int sizeY = 20;
	    int centerY = (int)(sizeY/2f);
	    scaleBar = createImage((int) sizeX, sizeY);
	    //figure out the scale
	    //float scaleVal = (sizeX - (xInset *2)) * scalex;

	    Graphics barGraph = scaleBar.getGraphics();
	    barGraph.drawLine(xInset,centerY,(int) sizeX-xInset,centerY);
	    barGraph.drawLine(xInset,0,xInset,sizeY);
	    barGraph.drawLine((int) sizeX-xInset,0,(int) sizeX-xInset,sizeY);

	    String zstr = scaleVal + " ft";
            FontMetrics fm = barGraph.getFontMetrics();
	    barGraph.drawString(zstr, (int) (sizeX - fm.stringWidth(zstr))/2, centerY - 1);
	    //System.out.println("Bar Redraw");	    
	}

        // Draw the (relatively) static background image
	if (backGround == null || !backSens || !bounds.equals(viewPort) || !backSize.equals(d)){
	    //rebuild the background image
	    ArrayList sensors = FDF.getSensors();
	    //System.out.println("Back Redraw");

	    // make the new image and get the graphics object
            backGround = createImage((int) bounds.getWidth(), 
				     (int) bounds.getHeight());
	    viewPort = bounds;
	    backSize = d;
	    Graphics2D backGraph = (Graphics2D)backGround.getGraphics();

            // Draw a grid based on scale bar size
            backGraph.setColor(new Color(128, 128, 128));
            for (float w = 0; w < envWidth; w += scaleVal) {
                for (float h = 0; h < envHeight; h += scaleVal) {
		    float trueX = w * scalex;
		    float trueY = h * scaley;		    
                    if (bounds.contains(trueX, trueY)) {
                        backGraph.draw(new Rectangle2D.Float(trueX - bounds.x, trueY - bounds.y, 1, 1));
                    }
                }
            }

	    // Draw the information
            backGraph.setColor(sensColor);
	    if (sensors != null) {
		for(Iterator i = sensors.iterator();i.hasNext();){
		    SensorModel SM = (SensorModel) i.next();
		    Point2D.Float pt = SM.getPoint(true);
		    if (pt == null) {
			backSens = false;
			return;
		    }
		    float trueX = (float)pt.getX() * scalex;
		    float trueY = (float)pt.getY() * scaley;		    
                    if (bounds.contains(trueX, trueY)) {
                        if (sensortext) {
                            int fontwidth = metrics.stringWidth(SM.getName());
                            backGraph.drawString(SM.getName(),
                                                 trueX - bounds.x - fontwidth/2,
                                                 trueY - bounds.y + fontheight/2);
                        } else {
                            backGraph.fill(new Rectangle2D.Float(trueX - bounds.x - (baseSensSize/2), 
                                                                 trueY - bounds.y - (baseSensSize/2),
                                                                 baseSensSize, baseSensSize));
                        }
		    }
		}
		backSens = true;
	    }
	}
	
	g.clipRect(bounds.x, bounds.y, bounds.width, bounds.height);
	g.drawImage(backGround, bounds.x, bounds.y, this);

	//how big should the target square be?
	float targXSize = baseSensSize;
	float targYSize = baseSensSize;
	
        // Now the trackings
        ArrayList sensors = FDF.getSensors();
	HashMap targets = FDF.getTargets();
        if (showtracks && sensors != null && targets != null) {
            Stroke originalstroke = g2.getStroke();
            for(Iterator i = sensors.iterator();i.hasNext();){
                SensorModel SM = (SensorModel) i.next();
                SlotSchedule ss = SM.getSchedule();
                Vector tracks = ss.getTracks();
                if (tracks.isEmpty()) continue;
		//System.err.println(SM.getName() + " Drawing schedule " + ss.toString());
                Point2D.Float pt1 = SM.getPoint(true);
                float sensX = (float)pt1.getX() * scalex;
                float sensY = (float)pt1.getY() * scaley;

                if (ss.hasCobinding()) {
                    g.setColor(Color.pink);
                } else {
                    g.setColor(Color.yellow);
                }

                for (Enumeration e = tracks.elements(); e.hasMoreElements();) {
                    String targetname = (String)e.nextElement();
                    TargetModel target = (TargetModel) targets.get(targetname);
                    Point2D.Float pt2 = target.getPoint(true);
                    float targX = (float)pt2.getX() * scalex;
                    float targY = (float)pt2.getY() * scaley;
                    if (bounds.contains(sensX, sensY) &&
                        bounds.contains(targX, targY)) {
                        int weight = ss.getCommitmentsSlots(targetname) * 2;
                        g2.setStroke(new BasicStroke(weight));
                        g2.draw(new Line2D.Float(sensX, sensY,
                                                 targX, targY));
                    }
                }
            }
            g2.setStroke(originalstroke);
        }

	// Now do the targets
	if (targets != null){
	    //g.setColor(targColor);
	    for(Iterator i = targets.values().iterator();i.hasNext();){
		TargetModel TM = (TargetModel) i.next();
		Point2D.Float pt = TM.getPoint(true);
		String state = TM.getState();

		if(pt == null) continue;
		float trueX = (float)pt.getX() * scalex;
		float trueY = (float)pt.getY() * scaley;
		
		if (bounds.contains(trueX, trueY)) {
		    if (state == null || state.equals("N") || !shownegot || state.equals("WAIT")) {
			if(state != null && state.equals("WAIT"))
			    g.setColor(waitColor);
			else
			    g.setColor(targColor);
		    } 
		    else{			
			g.setColor(negColor);
			StringTokenizer tok = new StringTokenizer(state, ",");
			while(tok.hasMoreTokens()){
			    String name = tok.nextToken();
			    TargetModel other = (TargetModel) targets.get(name);
			    if(other == null) continue;
			    Point2D.Float pt2 = other.getPoint(true);
			    if(pt2 == null) continue;
			    g2.draw(new Line2D.Float(trueX, trueY,
                                                     (float)pt2.getX() * scalex,
                                                     (float)pt2.getY() * scaley));
			}
		    }

                    if (targettext) {
                        int fontwidth = metrics.stringWidth(TM.getName());
                        g2.drawString(TM.getName(), trueX - fontwidth/2, trueY + fontheight/2);
			//System.err.println("Drawing: " + TM.getName() +" at " +trueX + "," + trueY);
                    } else {
                        g2.fill(new Ellipse2D.Float(trueX - targXSize/2, trueY - targYSize/2, 
                                                    targXSize, targYSize));
                    }
		}	
	    }
	    g.setColor(c);
	}

	//draw the scale bar
	g.drawImage(scaleBar, bounds.x, bounds.y+bounds.height-25, this);


    } 

    public void updateDisplay(){
	//System.err.println("Redraw");
	repaint();
    }

    public Dimension getPreferredSize() {
	//find the maximum extentx and extent y and return them 
	//or minimum of 600  for each
	int maxExX = 600;
	int maxExY = 600;
	return new Dimension(maxExX*zoomLevel, maxExY*zoomLevel);
    }
    
    
    
    public Dimension getMinimumSize() {return getPreferredSize(); }
    public Dimension getMaximumSize() {return getPreferredSize(); }

    private float Closest(double num, int precision){
	double scale;
	
	scale = Math.pow(10, precision);
	return (float) (Math.round(num / scale) * scale);
    }

} // FarmWorldPanel
