package farm.bandit.agent;

import farm.simulator.*;
import farm.metagent.*;

import java.util.*;
import utilities.Log;

/**
 */
public class ReinforcementComparisonAgent extends BanditAgent {
    public static String NAME = "rc";
    protected double alpha = 0.1;
    protected double p[];
    protected double pi[];
    protected double rbar;

    /**
     * Constructor
     * @param metagent A handle to the metagent this agent belongs to
     * @param name The agent's name
     */
    public ReinforcementComparisonAgent(FarmMetAgent metagent, String name) {
	super(metagent, name);
    }

    /**
     * Constructor
     * @param metagent A handle to the metagent this agent belongs to
     * @param alpha The alpha value to use
     * @param name The agent's name
     */
    public ReinforcementComparisonAgent(FarmMetAgent metagent, String name, double alpha) {
        this(metagent, name);

        this.alpha = alpha;
    }

    /**
     * Initialize the state vectors
     */
    public void begin() {
        super.begin();

        p = new double[getNumArms()];
        pi = new double[getNumArms()];
        rbar = 0;
        for (int i = 0; i < p.length; i++) {
            p[i] = 0;
        }
        for (int i = 0; i < p.length; i++) {
            pi[i] = pi(i);
        }
    }

    /**
     */
    public void pulse() {
        super.pulse();

        // First pick an arm
        float c = random.nextFloat();
        int arm = 0;
        for (int i = 0; i < p.length; i++) {
            arm = i;
            if (c < pi[arm]) {
                break;
            }
            c -= pi[i];
        }

        // Then pull it
        double reward = pullArm(arm);

        // Update p
        p[arm] = p[arm] + alpha * (reward - rbar);

        // Update rbar
        rbar = rbar + (1f / (float)getNumPulls()) * (reward - rbar);

        // Update pi
        float totalpi = 0;
        for (int i = 0; i < p.length; i++) {
            pi[i] = pi(i);
            totalpi += pi[i];
        }

        //System.err.println(getName() + " Arm:" + arm + " Reward:" + reward + " Optimal:" + isOptimalArm(arm) + " p:" + p[arm] + " pi:" + pi[arm] + " rbar:" + rbar + " totalpi:" + totalpi);
    }

    /**
     * Calculates the probability for a particular arm using
     * a Gibb's distribution
     * @param a The arm in question
     * @return The probability that arm should be selected
     */
    public double pi(int a) {

        double n = Math.exp(p[a]);
        double d = 0;

        for (int i = 0; i < p.length; i++) {
            d += Math.exp(p[i]);
        }

        return (n / d);
    }
}
