package farm.bandit.plugins; 

import java.rmi.*;
import java.rmi.server.*;

import farm.simulator.*;

import java.util.*;
import java.io.*;
import java.awt.geom.*;
import utilities.Log;
import farm.bandit.agent.*;

/**
 * This component instantiates the experiment.  It does
 * this by first determining how many of each type of agent
 * should be run.  Then, using the number of metagnent that
 * exist, it determines where these agents should be assigned.
 * <p>
 * Properties:<dl>
 * <dt><tt>-agents [integer]</tt>
 * <dd>The number of agents to run of each type (default 2000)
 * </dl>
 */
public class BanditDriver extends FarmPlugIn {
    int agents = 20;
    int metaAgents = 1;

    // Constants
    /**
     * Command line identifier for the number of variables
     */
    public static final String CLA_AGENTS = "agents";

    /**
     * Property identifier for number of metagent in the environmemt
     */
    public static final String CLA_METAGENTS = "metagents";

    /**
     * Constructor
     * @param args An array of command line arguments for the driver
     */
    public BanditDriver(String []args) throws RemoteException {
	super(args);

        // Command line args
        setPlugInName(getCommandLineArgument(args, CLA_NAME, "Bandit Driver"));
        setMetaAgents(Integer.valueOf(getCommandLineArgument(args, CLA_METAGENTS, String.valueOf(metaAgents))));
        setAgents(Integer.valueOf(getCommandLineArgument(args, CLA_AGENTS, String.valueOf(agents))));
    }

    /**
     * Sets the number of metagents
     * @param i The number of metagents
     */
    public void setMetaAgents(Integer i) {
        metaAgents = i.intValue();
        setProperty(CLA_METAGENTS, i);
    }

    /**
     * Sets the number of agents to use
     * @param i The number of agents
     */
    public void setAgents(Integer i) {
        agents = i.intValue();
        setProperty(CLA_AGENTS, i);
    }
                                                                                                                    
    /**
     * In this function we specify which species of learning
     * agents will be instantiated.  We also pass in variables to
     * those agents.
     */
    public void begin() throws RemoteException {
	super.begin();

        // Assign the agents
        log.log("Creating assignments for " + metaAgents + " metagents", Log.LOG_INFO);

        addAgents(eGreedyAgent.NAME + ",0.1,0", agents);
        //addAgents(eGreedyAgent.NAME + ",0.1,5", agents);
        //addAgents(PursuitAgent.NAME + ",0.005", agents);
        //addAgents(PursuitAgent.NAME + ",0.01", agents);
        //addAgents(PursuitAgent.NAME + ",0.05", agents);
        //addAgents(ReinforcementComparisonAgent.NAME + ",0.1", agents);
        //addAgents(ReinforceAgent.NAME + ",0.1,0.1,1", agents);
        //addAgents(ReinforceAgent.NAME + ",0.01,0.1,1", agents);
        //addAgents(ReinforceAgent.NAME + ",0.05,0.1,1", agents);
    }

    /**
     * This function distributes the agents among the
     * available metagents.
     * @param type The type of agent
     * @param num How many of those agent to spawn
     */
    public void addAgents(String type, int num) {
        int div = (int)Math.floor(num / metaAgents);
        int mod = num % metaAgents;

        log.log("Adding " + num + " " + type, Log.LOG_INFO);
        for (int i = metaAgents-1, start = 0; i >= 0; i--) {
            String metkey = "MA" + i  + farm.metagent.FarmMetAgent.AGENTS_DESCRIPTION;
            int finish = start + div-1;
            if (mod > 0) {
                finish++;
                mod--;
            }
            if (finish >= start) {
                String assign = type + " " + start + "-" + finish;
                String prev = (String)getProperty(metkey);
                if (prev != null)
                    assign = prev + " " + assign;
                setProperty(metkey, assign);
            }
            start = finish + 1;
        }
    }

    /**
     * Main Function
     */
    public static void main(String[] args) {
	try {
	    FarmPlugIn plugin = new BanditDriver(args);
	    plugin.run();
	} catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
        }
    }
}
