package farm.generic.plugins;

import java.rmi.*;
import java.rmi.server.*;

import farm.simulator.*;

import java.util.*;

/**
 * A simple analysis function, which can be used to augment the
 * analytic functions of a more derived class.
 */
public class AnalysisFunction implements LocalFunction {

    // Constants
    public static final String PROP_TOTALMESSAGESSENT = "TotalMessagesSent";
    public static final String PROP_TOTALMESSAGESRECEIVED = "TotalMessagesReceived";
    public static final String PROP_TOTALBYTESSENT = "TotalBytesSent";
    public static final String PROP_TOTALBYTESRECEIVED = "TotalBytesReceived";

    public Object run(PropertyOwner owner) {

        countMessagesSent(owner);
        countMessagesReceived(owner);
        countBytesSent(owner);
        countBytesReceived(owner);

        return null;
    }

    /**
     * Counts the messages sent in the simulation's current time interval
     * @param owner The owner's reference
     * @return the number of messages sent
    */
    public int countMessagesSent(PropertyOwner owner) {
        int msgsent = 0;
	
        try {
            Collection list = owner.findProperties(":MessagesSent");
            for (Iterator i = list.iterator(); i.hasNext(); ) {
                String name = (String)i.next();
                Number num = (Number)owner.getProperty(name);
                msgsent += num.intValue();
            }
            owner.setProperty(PROP_TOTALMESSAGESSENT, new Integer(msgsent));
	    
        } catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
        }
	return msgsent;
    }

    /**
     * Counts the bytes sent in the simulation's current time interval
     * @param owner The owner's reference
     * @return the number of bytes sent
    */
    public int countBytesSent(PropertyOwner owner) {
        int byteSent = 0;
	
        try {
            Collection list = owner.findProperties(":BytesSent");
            for (Iterator i = list.iterator(); i.hasNext(); ) {
                String name = (String)i.next();
                Number num = (Number)owner.getProperty(name);
                byteSent += num.intValue();
            }
            owner.setProperty(PROP_TOTALBYTESSENT, new Integer(byteSent));
	    
        } catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
        }
	return byteSent;
    }


    /**
     * Counts the messages received during the simulation's current time interval
     * @param owner The owner's reference
     * @return the number of messages recieved 
    */
    public int countMessagesReceived(PropertyOwner owner) {
        int msgrecv = 0;
	
        try {
            Collection list = owner.findProperties(":MessagesReceived");
            for (Iterator i = list.iterator(); i.hasNext(); ) {
                String name = (String)i.next();
                Number num = (Number)owner.getProperty(name);
                msgrecv += num.intValue();
            }
            owner.setProperty(PROP_TOTALMESSAGESRECEIVED, new Integer(msgrecv));
	    
        } catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
        }
	return msgrecv;
    }

    /**
     * Counts the bytes received during the simulation's current time interval
     * @param owner The owner's reference
     * @return the number of bytes recieved 
    */
    public int countBytesReceived(PropertyOwner owner) {
        int bytesrecv = 0;
	
        try {
            Collection list = owner.findProperties(":BytesReceived");
            for (Iterator i = list.iterator(); i.hasNext(); ) {
                String name = (String)i.next();
                Number num = (Number)owner.getProperty(name);
                bytesrecv += num.intValue();
            }
            owner.setProperty(PROP_TOTALBYTESRECEIVED, new Integer(bytesrecv));
	    
        } catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
        }
	return bytesrecv;
    }

}
