package farm.graphcolor.plugins;

import java.rmi.*;
import java.rmi.server.*;

import utilities.Log;
import farm.simulator.*;
import farm.generic.plugins.*;
import farm.generic.plugins.drivers.*;
import farm.graphcolor.plugins.drivers.*;

import java.util.*;

/**
 */
public class GraphClusterAnalysis extends FarmPlugIn {

    /**
     * Constructor
     * @param address The address of the remote simulator to eonnect to
     */
    public GraphClusterAnalysis(String []args)throws RemoteException {
        super(args);

        // Command line args
        setPlugInName(getCommandLineArgument(args, CLA_NAME, "Graph Color Analysis"));
    }

    /**
     * This is called at the end of a pulse.
     */
    public void pulseEnd() {

        try {
            LocalFunction function = new GraphClusterAnalysisFunction();            
            //simulator.runLocalFunction(new GraphColorAnalysisFunction());
            function.run(simulator);

        } catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * The main function
     */
    public static void main(String[] args) {

        try {
            FarmPlugIn plugin = new GraphClusterAnalysis(args);
            plugin.run();
        } catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
        }
    }
}

class GraphClusterAnalysisFunction extends GraphColorAnalysisFunction {

    public static final String PROP_INTRA = "IntraLinks";
    public Object run(Simulator simulator) {
	super.run(simulator);

        try {
	    ArrayList clusters = (ArrayList) simulator.getProperty(GraphColorDriver.PROP_CLUSTERS);
	    int totalLinks = 0;
	    int intraLinks = 0;
	    for(Iterator i = nodes.iterator();i.hasNext();){
		String nName = (String) i.next();
		ArrayList links = (ArrayList) simulator.getProperty(nName + ":links");
		ArrayList neighs = (ArrayList)simulator.getProperty(nName + ":neighbors");

		Integer inName = new Integer(nName.substring(2));
		//find this nodes cluster
		ArrayList clust = null;
		for(Iterator j = clusters.iterator();j.hasNext();){
		    ArrayList thisClust = (ArrayList) j.next();
		    if(thisClust.contains(inName)){
			clust = thisClust;
			break;
		    }
		}

//		System.err.println("neighs " + ((neighs == null)?"NULL":neighs.toString()) + " links " + ((links ==null)?"NULL":links.toString()));

		if(links != null && neighs != null && clust != null){ 
		    for(Iterator j = links.iterator();j.hasNext();){
			String oName = (String) j.next();
			Integer ioName = new Integer(oName.substring(2));
			if(!neighs.contains(oName) && !nName.equals(oName)){
			    totalLinks ++;
			    if(clust.contains(ioName)) 
				intraLinks++;
			}
		    }
		}
	    }

	    System.err.println("intraLinks " + intraLinks + " totalLinks " + totalLinks);
		float perc = 0.0f;
		if(totalLinks >0) perc = (float) intraLinks/ (float) totalLinks;    

	    simulator.setProperty(PROP_INTRA, new Float(perc));

        } catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
        }

        return null;
    }
}
