package farm.graphcolor.plugins;

import java.rmi.*;
import java.rmi.server.*;

import utilities.Log;
import farm.simulator.*;
import farm.generic.plugins.*;
import farm.generic.plugins.drivers.*;
import farm.graphcolor.plugins.drivers.*;

import java.util.*;

/**
 */
public class GraphColorAnalysis extends FarmPlugIn {

    /**
     * Constructor
     * @param address The address of the remote simulator to eonnect to
     */
    public GraphColorAnalysis(String []args)throws RemoteException {
        super(args);

        // Command line args
        setPlugInName(getCommandLineArgument(args, CLA_NAME, "Graph Color Analysis"));
    }

    /**
     * This is called at the end of a pulse.
     */
    public void pulseEnd() {

        LocalFunction function = new GraphColorAnalysisFunction();            
        /*try {
            //simulator.runLocalFunction(new GraphColorAnalysisFunction());
            function.run(simulator);

        } catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
            }*/
        ((GraphColorAnalysisFunction)function).run(this);
    }

    /**
     * The main function
     */
    public static void main(String[] args) {

        try {
            FarmPlugIn plugin = new GraphColorAnalysis(args);
            plugin.run();
        } catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
        }
    }
}

class GraphColorAnalysisFunction extends AnalysisFunction {

    // Constants
    public static final String PROP_TOTALNODES = "TotalNodes";
    public static final String PROP_TOTALEDGES = "TotalEdges";
    public static final String PROP_POSSIBLECONFLICTS = "PossibleConflicts";
    public static final String PROP_TOTALCONFLICTS = "TotalConflicts";
    public static final String PROP_TOTALLINKS = "TotalLinks";
    public static final String PROP_SOLVED = "Solved";
    public static final String PROP_MAXLINKS = "MaxLinks";
    public static final String PROP_MAXLINKINC = "MaxLinkInc";
    public static final String PROP_AVGLINKINC = "AvgLinkInc";
    public static final String PROP_NUMAGENTINC = "NumAgentInc";

    protected ArrayList nodes = new ArrayList(); 
    int lastTime = 0;
    public Object run(Simulator simulator) {
        return null;
    }

    public Object run(PlugIn plugin) {
        int numedges = 0;
        try {
	    if(plugin.getProperty(Simulator.P_TIME) == null) return null;
	    int time = ((Integer) plugin.getProperty(Simulator.P_TIME)).intValue();

	    if(lastTime == time) return null;
            // Count sensors
            Hashtable edges = new Hashtable();
            Hashtable colors = new Hashtable();
            Collection list = plugin.findProperties(":color");
            for (Iterator i = list.iterator(); i.hasNext(); ) {
                String name = (String)i.next();
                if (name.startsWith("ND")) {
                    name = name.substring(0, name.indexOf(":"));
                    nodes.add(name);
		    Hashtable e = (Hashtable)plugin.getProperty(name + ":neighbors");
                    if (e != null) {
                        numedges += e.size();
                        edges.put(name, e);
                    }
                    Integer c = (Integer)plugin.getProperty(name + ":color");
                    colors.put(name, c);
		}
            }

            plugin.setProperty(PROP_TOTALNODES, new Integer(nodes.size()));
            plugin.setProperty(PROP_TOTALEDGES, new Integer(numedges / 2));

            // Track messaging
            int msgsent = countMessagesSent(plugin);
            int msgrcv = countMessagesReceived(plugin);
	    //int bytesent = countBytesSent(simulator);
            //int bytercv = countBytesReceived(simulator);

            // Determine number of possible conflicts
            int pconflicts = 0,
                tconflicts = 0;
	    Vector cons = new Vector();
            for (Enumeration en = edges.keys(); en.hasMoreElements(); ) {
                String name = (String)en.nextElement();

                Hashtable e = (Hashtable)edges.get(name);

                Integer acolor = (Integer)colors.get(name);
                for (Iterator i = e.keySet().iterator(); i.hasNext(); ) {
                    String name2 = (String) i.next();
		    if(Integer.parseInt(name.substring(2)) > Integer.parseInt(name2.substring(2))){
			
			int value = ((Integer) e.get(name2)).intValue();
			pconflicts += value;
			Integer bcolor = (Integer)colors.get(name2);
			if (acolor.intValue() == -1 || bcolor.intValue() == -1)
			    tconflicts += value;
			else if (acolor.equals(bcolor)) {
			    tconflicts += value;
			    cons.add(name + "-" + name2);  
			}
		    }
                }
            }
            plugin.setProperty(PROP_POSSIBLECONFLICTS, new Integer(pconflicts));
            plugin.setProperty(PROP_TOTALCONFLICTS, new Integer(tconflicts));

	    //determine the linking
	    int numLinks = 0;
	    int maxLinks = 0;
	    int maxLinkInc = 0;
	    int avgLinkInc = 0;
	    int numAgentInc = 0;
	    for(Iterator i = nodes.iterator();i.hasNext();){
		String nName = (String) i.next();
		ArrayList links = (ArrayList) plugin.getProperty(nName + ":links");
		Hashtable neighs = (Hashtable)plugin.getProperty(nName + ":neighbors");
		if(links != null && neighs != null){ 
		    numLinks += links.size() - 1;
		    if(links.size() > maxLinks) maxLinks = links.size();
		    int inc = links.size() - neighs.size() -1;
		    if(inc > 0) numAgentInc ++;
		    avgLinkInc += inc;
		    if(inc > maxLinkInc) maxLinkInc = inc;
		}
	    }
	    if( numAgentInc>0)
		avgLinkInc /= numAgentInc;
	    plugin.setProperty(PROP_TOTALLINKS, new Integer(numLinks));
	    plugin.setProperty(PROP_MAXLINKS, new Integer(maxLinks));
	    plugin.setProperty(PROP_MAXLINKINC, new Integer(maxLinkInc));
	    plugin.setProperty(PROP_AVGLINKINC, new Float(avgLinkInc));
	    plugin.setProperty(PROP_NUMAGENTINC, new Integer(numAgentInc));

            // Print the colors
	    System.err.print(time + " ");
            for (int i = 0; i < colors.size(); i++) {
                String key = "ND" + i;
                System.err.print(colors.get(key));
            }
            System.err.print(" - " + tconflicts + "\n");

            // Print the cons
            for (int i = 0; i < cons.size(); i++) {
                System.err.print((String) cons.elementAt(i));
		if(i<cons.size()-1){
		    System.err.print(" ");
		}
	    }
            System.err.print("\n");
	    int opt = ((Integer) plugin.getProperty("Optimal")).intValue();
	    boolean done = true;
	    for(Iterator i = nodes.iterator();i.hasNext();){
		String nName = (String) i.next();
		if(plugin.getProperty(nName + ":done") != null){
		    boolean thisDone = ((Boolean) plugin.getProperty(nName + ":done")).booleanValue();
		    if(done) done = thisDone;
		    if(thisDone) System.err.print("..." + nName);
		}
		else{
		    done = false;
		}
	    }
	    System.err.print("\n");
 


	    if (msgrcv == 0 && msgsent == 0 && done){
		//if(tconflicts == opt){
		plugin.setProperty(StopDriver.STOPSIMULATION, new Boolean(true));
		plugin.setProperty(PROP_SOLVED, new Boolean(true));
		//}
            }

	    lastTime = time;


        } catch (RemoteException e) {
            System.err.println("Exception: " + e.getMessage());
            e.printStackTrace();
        }

        return null;
    }
}
