package farm.graphcolor.plugins.display; 
import java.rmi.*;
import java.rmi.server.*;
import farm.simulator.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;
import farm.graphcolor.plugins.drivers.*;
/**
 * Shows a display of the graph color environment.  This GUI primarily shows 
 * sensors (their current color) and edges (black if 
 * constraint satidfied, red otherwise) in the environment.  It will
 * also show negotiations in progress (blue lines). 
 */
public class GraphDisplay extends FarmPlugIn {
    
    int width = 600;
    int height = 600;
    boolean startUp = true;
    int numNodes = 50;
    ArrayList nodes = new ArrayList();
    GraphDisplayFrame display = null;
    boolean started = false;
    int nodeSpacing = 50;
    int baseNodeSize = 8;

    /**
     * Constructor
     */
    public GraphDisplay(String []args)throws RemoteException {
        super(args);

        // Command line args
        setPlugInName(getCommandLineArgument(args, CLA_NAME, "Graph Color Display"));
    }

    /**
     * This is called at the beginning of a pulse, before the
     * agents are started.
     */
    public void pulseBegin() throws RemoteException {
	if (startUp) {
	    if (getProperty(GraphColorDriver.PROP_DRIVERSTARTUP) == null) {
		return;
	    } else {
		//we can do something
		//System.err.println("Have data to get");
		startUp = false;

		numNodes = ((Integer) getProperty(GraphColorDriver.CLA_NODES)).intValue();

		display.setWidth(width);
		display.setHeight(height);
		display.setNodes(createNodes(numNodes));
	    }
	} else {
	    //update the targets locations
            if (timeChanged()) {
                display.updateDisplay();
            }
	}
    }

    /**
     * This is called at the end of a pulse, after the agents
     * should have completed.
     */
    public void pulseEnd() {
    }

    /**
     */
    private ArrayList createNodes(int numNodes) {
	for(int i = 0;i<numNodes;i++){
	    String name = "ND" + i;
	    Hashtable neighbors = (Hashtable) getProperty(name + ":neighbors");
	    NodeVisual NV = new NodeVisual(name, baseNodeSize,baseNodeSize);
	    for(Iterator neigh = neighbors.keySet().iterator(); neigh.hasNext();){
		NV.addLink((String) neigh.next(), "!=");
	    }
	    nodes.add(NV);
	}
	//	setInitialPoints(nodes, (NodeVisual) nodes.get(0), null);
	setInitialPoints(nodes);
	//for(int i = 0;i<numNodes;i++)
	//System.err.println(((NodeVisual) nodes.get(i)).toString());
	return nodes;
    }

    /**
     * Sets up the initial location of the nodes using the radial method with
     * the root being the mode with the highest degree
     */
    private void setInitialPoints(ArrayList nodes, NodeVisual root, NodeVisual parent){
	//pick the first one and place it
	ArrayList lns = root.getLinks();
	int startx,starty;
	int parentx, parenty;
	double ddegrees; double startd;
	if(parent != null){
	    startx = root.getX();
	    starty = root.getY();
	    parentx = parent.getX();
	    parenty = parent.getY();
	    ddegrees = (double) 2*Math.PI/(lns.size() + 1);
	    if(parentx != startx){
		startd = Math.atan((parenty-starty)/(parentx-startx)); 
	    }
	    else{
		startd =0.0;
	    }
	}
	else{
	    //first node to place, put it in the middle
	    root.setX((int) (width/2));
	    root.setY((int) (height/2));
	    root.setDone();
	    startx = root.getX();
	    starty = root.getY();
	    ddegrees = (double) 2*Math.PI/(lns.size());
	    startd = 0.0;
	}
	
	for(Iterator i = lns.iterator();i.hasNext();) {
	    NodeLink ln = (NodeLink) i.next();

	    NodeVisual TMV = null;
	    for(Iterator j = nodes.iterator();j.hasNext();){
		NodeVisual n = (NodeVisual) j.next();
		if(n.getName().equals(ln.getWho())){
		    TMV = n;
		    break;
		}
	    }

	    if(!TMV.done){
		startd += ddegrees;
		TMV.setX((int) (startx + nodeSpacing* Math.cos(startd)));
		TMV.setY((int) (starty + nodeSpacing* Math.sin(startd)));
		TMV.setDone();
	    }
	}

	for(Iterator i = lns.iterator();i.hasNext();) {
	    NodeLink ln = (NodeLink) i.next();

	    if(!ln.done()){
		ln.setDone();
		NodeVisual TMV = null;
		for(Iterator j = nodes.iterator();j.hasNext();){
		    NodeVisual n = (NodeVisual) j.next();
		    if(n.getName().equals(ln.getWho())){
			TMV = n;
			break;
		    }
		}
		setInitialPoints(nodes, TMV, root);
	    }
	}
    }


    private void setInitialPoints(ArrayList nodes){
	int numx = (int)Math.ceil(Math.pow(nodes.size(), (float)width / (float)(width + height)));
	int numy = (int)Math.ceil(Math.pow(nodes.size(), (float)height / (float)(width + height)));
	float xdelta = (float)width / numx;
	float ydelta = (float)height / numy;

        float x = 0, y = -ydelta / 2;	
	for(int i = 0;i<nodes.size();i++){
	    NodeVisual n = (NodeVisual) nodes.get(i);
	    if (i % numx == 0) {
		x = xdelta / 2;
		y += ydelta;
	    } else {
		x += xdelta;
	    }
	    n.setX((int) x);
	    n.setY((int) y);
 	}
    }



    /**
     */
    public void begin() throws RemoteException {
	super.begin();
	display = new GraphDisplayFrame(this);
	display.pack();
	display.show();
    }

    /**
     * The main function
     */
    public static void main(String[] args) {

        try {
            GraphDisplay disp = new GraphDisplay(args);
            disp.run();

        } catch (Exception e) {
            System.err.println("Bind exception: " + e.getMessage());
            e.printStackTrace();
        }
        System.exit(0);
    }
} // GraphDisplay
