package farm.graphcolor.plugins.display;
/**
 * NodeVisual.java
 *
 *
 * Created: Tue Dec 11 15:45:46 2001
 *
 * @author Roger Mailler
 * @version
 */

import java.awt.*;
import java.util.*;

public class NodeVisual implements Comparable{
    String name;
    ArrayList links = new ArrayList();
    int x , y, height, width;
    boolean done = false;
    boolean hot = false;
    int color = 0;
    boolean selected = false;
    float textHeight = -1, textWidth = -1;
    boolean textMode = false;
    Color colors[] = new Color[10];

    NodeVisual(String n, int w, int h){
	name = n;
	height = h;
	width = w;
	
	colors[0] = Color.black;
	colors[1] = Color.red;
	colors[2] = Color.blue;
	colors[3] = Color.magenta;
	colors[4] = Color.yellow;
	colors[5] = Color.orange;
	colors[6] = Color.green;
	colors[7] = Color.pink;
	colors[8] = Color.cyan;
	colors[9] = Color.darkGray;
    }

    void addLink(String who, String What){
	//check to make sure we have a connection with this guys
	NodeLink newLink = new NodeLink(who);
	newLink.setConstraint(What);
	links.add(newLink);
    }

    public void removeLink(String who){
	NodeLink NL = null;
	for(Iterator i = links.iterator();i.hasNext();){
	    if(((NodeLink) i.next()).getWho().equals(who)){
		i.remove();
		return;
	    }
	}
    }

    void setLinkHot(String who, Vector whatHot){
	//find the link and make it hot
	for(Iterator i = links.iterator();i.hasNext();){
	    NodeLink NL = (NodeLink) i.next();
	    if(NL.getWho().equals(who)){
		//found it
		NL.setHot();
		break;
	    }
	}
    }

    void resetLinks(){
	for(Iterator i = links.iterator();i.hasNext();){
	    ((NodeLink) i.next()).resetDone();
	}
    }
    void resetLinksHot(){
	for(Iterator i = links.iterator();i.hasNext();){
	    ((NodeLink) i.next()).resetHot();
	}
    }

    void resetDone(){done = false;}
    void setDone(){done = true;}
    boolean done(){return done;}
    String getName(){return name;}
    ArrayList getLinks(){return links;}
    public String toString(){
	String ans;
	ans = "Name: " + name + " x: " + getX() + " y: " + getY() + "\n";
	for(Iterator i = links.iterator();i.hasNext();){
	    ans = ans + ((NodeLink) i.next()).toString()+ "\n";
	}
	return ans;
    }
    int numLinks(){return links.size();}
    void setX(int val){x = val;}
    void setY(int val){y = val;}
    int getX(){return x;}
    int getY(){return y;}   
    void setHeight(int val){height = val;}
    void setWidth(int val){width = val;}
    int getHeight(){return height;}
    int getWidth(){return width;}    
    void setNodeColor(int c){color = c;}
    int getNodeColor(){return color;}

    int getExtentX(){return x + width;}
    int getExtentY(){return y + height;}
    void setSelected(){selected = true;}
    void resetSelected(){selected=false;}
    boolean isSelected(){return selected;}

    public boolean getTextMode(){return textMode;}
    public void setTextMode(boolean mode){textMode = mode;}

    public boolean getHot(){return hot;}
    public void setHot(boolean heat){hot = heat;}


    void draw(Graphics g){
	draw(g, 1.0f);
    }
    void draw(Graphics g, float zoomLevel){
	//draw the node
	Graphics2D g2 = (Graphics2D) g;
	
	//set the color
	Color c = g2.getColor();
	g2.setColor(colors[color]);

	float drawX = x * zoomLevel;
	float drawY = y * zoomLevel;

	Font saveFont = g2.getFont();

	if(!textMode){
	    //draw the node
	    if(hot){
		g2.drawRect((int) (drawX -  width/2) -2, (int) (drawY -  height/2) -2, width+3, height+3);
	    }
	    g2.fillRect(((int) drawX -  width/2), ((int) drawY -  height/2), (int) width, height);
	}
	else{
	    //draw the agent name
	    Font newFont = saveFont.deriveFont(saveFont.getSize()+4);
	    g2.setFont(newFont);
	    if(textHeight == -1){
		textHeight = g2.getFontMetrics().getAscent() + g2.getFontMetrics().getDescent();
		textWidth =  g2.getFontMetrics().stringWidth(name);
	    }
	    g2.drawString(name, ((int) drawX - textWidth/2), ((int) drawY + textHeight/2));
	    if(hot){
		g2.drawRect((int) (drawX -  textWidth/2) -2, (int) (drawY -  textHeight/2)+1, (int) textWidth+3, (int) textHeight+3);
	    }
	    g2.setFont(saveFont);
	}
    }

    boolean isInside(int x, int y, float zoomLevel){
	float drawX = this.x * zoomLevel;
	float drawY = this.y * zoomLevel;
	int wd2,hd2;
	if(textMode){
	    wd2 = (int) textWidth/2;
	    hd2 = (int) textHeight/2;
	}
	else{
	    wd2 = (int) width/2;
	    hd2 = (int) height/2;
	}
	return (x>=drawX-wd2 && x<=drawX+wd2 
		&& y>=drawY-hd2 && y<=drawY+hd2);
    }

    public boolean equals(Object o1){
	return numLinks() == ((NodeVisual) o1).numLinks();
    }

    public int compareTo(Object o1){
	NodeVisual oTMV = (NodeVisual) o1;
	int ans = 0;
	if(oTMV.numLinks() > this.numLinks()){
	    ans = -1;
	}
	else if(oTMV.numLinks() < this.numLinks()){
	    ans = 1;
	}
	return ans;
    }
} // NodeVisual
