package farm.sat.plugins.display;
/**
 * SATDisplayFrame.java
 *
 *
 * Created: Wed Jul 10 14:05:04 2002
 *
 * @author Roger Mailler
 * @version
 */
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.awt.print.*;

public class SATDisplayFrame extends JFrame
    implements ActionListener {
    
    String title = "SAT Display";
    JPanel JP = new JPanel();
    JScrollPane JSP;
    SATPanel topPanel = new SATPanel(this);
    JPanel bottomPanel = new JPanel();
    SATDisplay parent;
    JMenu menuFile;
    JMenuBar menubar;
    JMenuItem miAbout, miQuit, miPrint, miFreeze, miPageSetup;
    ArrayList nodes = null;
    PageFormat PF = new PageFormat();

    public SATDisplayFrame(SATDisplay p){
	setTitle(title);
	getContentPane().setLayout(new BorderLayout());
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    
		    System.exit(0);
		}
	    });
	parent = p;

        JSP = new JScrollPane();
        JSP.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        JSP.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        JSP.getViewport().add(topPanel);

        getContentPane().add(JP);
	JP.setLayout(new BorderLayout());
	JP.add(JSP, BorderLayout.CENTER);
	JP.add(bottomPanel, BorderLayout.SOUTH);
	
	buildMenus();
    }
    public void actionPerformed(ActionEvent e) {

	if (e.getSource() == miAbout) {
	    JOptionPane.showMessageDialog(null,
					  "SAT Display Plugin\n\nby Roger Mailler\n\nMulti-Agent Systems Lab\nUniversity of Massachusetts\nhttp://mas.cs.umass.edu/",
					  "About the Display",
					  JOptionPane.INFORMATION_MESSAGE);
	    
	} 
	else if (e.getSource() == miQuit)
	    System.exit(0);
	else if (e.getSource() == miPageSetup){
	    PrinterJob PJ = PrinterJob.getPrinterJob();
	    PF = PJ.pageDialog(PF);
	}
	else if (e.getSource() == miPrint) {
	    PrinterJob PJ = PrinterJob.getPrinterJob();
	    if(PJ.printDialog()){
		PJ.setPrintable(topPanel,PF);
		//PageFormat PF = PJ.pageDialog(new PageFormat());
		try{
		    PJ.print();
		}
		catch(PrinterException e1){
		    //do nothing
		}
	    }
	} 
	else if (e.getSource() == miFreeze) {
	    JOptionPane.showMessageDialog(null,
					  "Coming Soon!!!", "Information",
					  JOptionPane.INFORMATION_MESSAGE);
	    
	}

    }


    private void buildMenus(){
 	
	menubar = new JMenuBar();

	menuFile = new JMenu("File");
	this.setJMenuBar(menubar);
	menubar.add(menuFile);
	miAbout      = new JMenuItem("About");
	miQuit       = new JMenuItem("Quit");
	miPrint      = new JMenuItem("Print");
	miFreeze       = new JMenuItem("Freeze");
        miPageSetup  = new JMenuItem("Page Setup");


        menuFile.add(miPageSetup);
	menuFile.add(miPrint);
        menuFile.addSeparator();
	menuFile.add(miFreeze);
	menuFile.add(miAbout);
        menuFile.addSeparator();
	menuFile.add(miQuit);
	miAbout.addActionListener(this);
	miQuit.addActionListener(this);
	miPrint.addActionListener(this);
	miPageSetup.addActionListener(this);
	miFreeze.addActionListener(this);;

        JMenu menuPrefs = new JMenu("Preferences");
        menubar.add(menuPrefs);

        JCheckBoxMenuItem miNegot = new JCheckBoxMenuItem("Show Negotiations", topPanel.getShowNegotiations());
        miNegot.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JCheckBoxMenuItem item = (JCheckBoxMenuItem)e.getSource();
                    item.setSelected(item.isSelected());
                    topPanel.setShowNegotiations(item.isSelected());
                }
            });
        menuPrefs.add(miNegot);

        JCheckBoxMenuItem miNodeText = new JCheckBoxMenuItem("Show Node Text", topPanel.getShowNodeText());
        miNodeText.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JCheckBoxMenuItem item = (JCheckBoxMenuItem)e.getSource();
                    topPanel.setShowNodeText(!topPanel.getShowNodeText());
                    item.setSelected(topPanel.getShowNodeText());
                }
            });
        menuPrefs.add(miNodeText);

        JCheckBoxMenuItem miConstText = new JCheckBoxMenuItem("Show Constraint Text", topPanel.getShowConstText());
        miConstText.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JCheckBoxMenuItem item = (JCheckBoxMenuItem)e.getSource();
                    item.setSelected(item.isSelected());
                    topPanel.setShowConstText(item.isSelected());
                }
            });
        menuPrefs.add(miConstText);

        JCheckBoxMenuItem miConstLinks = new JCheckBoxMenuItem("Show Constraint Links", topPanel.getShowConstLinks());
        miConstLinks.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JCheckBoxMenuItem item = (JCheckBoxMenuItem)e.getSource();
                    item.setSelected(item.isSelected());
                    topPanel.setShowConstLinks(item.isSelected());
                }
            });
        menuPrefs.add(miConstLinks);

        JCheckBoxMenuItem miLinking = new JCheckBoxMenuItem("Show Agent Linking", topPanel.getShowLinking());
        miLinking.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JCheckBoxMenuItem item = (JCheckBoxMenuItem)e.getSource();
                    item.setSelected(item.isSelected());
                    topPanel.setShowLinking(item.isSelected());
                }
            });
        menuPrefs.add(miLinking);

    }

    public void updateDisplay(){
	topPanel.updateDisplay();
    }

    public void setNodes(ArrayList nods){
	nodes = nods;
	topPanel.updateDisplay();
    }

    public ArrayList getNodes(){
	return nodes;
    }

    public void setHeight(int h){
	topPanel.setEnvHeight(h);
    }
    public void setWidth(int w){
	topPanel.setEnvWidth(w);
    }

    public JScrollPane getJSP(){
	return JSP;
    }

    public Object getProperty(String prop){
	return parent.getProperty(prop);
    }


} // SATDisplayFrame
