/**
 */

package farm.simulator;

import java.util.*;
import java.io.Serializable;

/**
 */
public class DataToken implements Serializable {
    boolean isowner = false;
    String source = null;
    boolean trans = false;
    ArrayList cache = null;
    transient PlugIn plugin = null;

    // Owner
    public DataToken() {

        setOwner(true);
    }

    // Transient cache
    public DataToken(boolean t, String s) {

        setTransient(t);
        setSource(s);
    }

    // Remote source
    public DataToken(String s) {

        setSource(s);
    }

    public void setTransient(boolean t) { trans = t; }
    public boolean isTransient() { return trans; }

    public boolean isOwner() { return isowner; }
    public void setOwner(boolean o) { isowner = o; }

    public String getSource() { return source; }
    public void setSource(String s) { source = s; }
    public PlugIn getSourcePlugIn() { return plugin; }
    public void setSourcePlugIn(PlugIn p) { plugin = p; }
    
    public boolean hasCacheTargets() { return (cache == null) ? false : (cache.size() > 0); }
    public synchronized void addCacheTargets(Collection c) {
        if (c == null) return;
        if (cache == null) {
            cache = new ArrayList();
        }
        cache.addAll(c);
    }
    public synchronized void setCacheTargets(Collection c) {
        if (c == null) return;
        if (cache == null) {
            cache = new ArrayList();
        }
        cache.clear();
        cache.addAll(c);
    }
    public synchronized void addCacheTarget(String t) {
        if (cache == null) {
            cache = new ArrayList();
        }
        if (!cache.contains(t)) {
            cache.add(t);
        }
    }
    public synchronized void removeCacheTarget(String t) {
        if (cache == null) return;
        cache.remove(t);
    }
    public Collection getCacheTargetsCollection() {
        return cache;
    }
    public Iterator getCacheTargets() {
        if (cache == null) {
            return java.util.Collections.EMPTY_LIST.iterator();
        } else {
            return cache.iterator();
        }
    }

    public String toString() {
        return "O:" + isOwner()
            + " T:" + isTransient()
            + " S:" + getSource()
            + " C:" + cache;
    }
}
